use types::{token::Token, token::TokenType, Loc};



mod types;

pub struct Parser {
    tokens: Vec<Token>,
    chars: Vec<char>,
    loc: Loc,
}

impl Parser {
    pub fn new() -> Self {
        Self {
            tokens: Vec::new(),
            chars: Vec::new(),
            loc: Loc::default(),
        }
    }

    pub fn parse(&mut self, code: String, file_name: String) -> anyhow::Result<()> {
        self.loc.file = file_name;
        self.loc.col = 0;
        self.loc.line = 0;
        
        self.chars = code.chars().collect();
        self.chars.reverse();

        // Parse here :3

        while let Some(c) = self.next() {

            let typ = match c {
                '+' => TokenType::Add,
                '-' => TokenType::Sub,
                '<' => TokenType::Left,
                '>' => TokenType::Right,
                '[' => TokenType::LoopL,
                ']' => TokenType::LoopR,
                '.' => TokenType::Print,
                ',' => TokenType::Input,
                _ => continue
            };
        
            self.tokens.push(Token {
                typ,
                loc: self.loc.clone()
            });
        }  

        Ok(())
    }

    /// Pops one token
    fn next(&mut self) -> Option<char> {
        if let Some(c) = self.chars.pop() {
            match c {
                '\n' => {
                    self.loc.line += 1;
                    self.loc.col = 0;
                }
                _ => {
                    self.loc.col += 1;
                }
            }
            return Some(c);
        }

        None
    }
    
    pub fn tokens(&self) -> &[Token] {
        &self.tokens
    }

}